;;; template-mode.el --- minor mode for inserting templates

;;; Copyright (c) 2001, Brian St. Pierre <bstpierre @ bstpierre.org>

;;; Author: Brian St. Pierre
;;; Maintainer: Brian St. Pierre <bstpierre @ bstpierre.org>
;;; Keywords: template, C
;;; Created: 2000-01-01
;;; Modified: 2001-09-23 (v 1.0)

(defconst template-mode-version "1.0"
	"Template mode version number.")

;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or (at your option) any later version.

;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


;;;;;;; TODO ;;;;;;;;;;
;;;
;;; Allow users to put lisp code into the template, surrounded by
;;; delimiters. This code would not be inserted into the buffer on an
;;; insert, but arguments could be passed from the insertion function to the
;;; arbitrary code or the arbitrary code could prompt for further interactive
;;; customization.!!

(defvar template-mode nil)
(make-variable-buffer-local 'template-mode)

;;
;; These can be modified in your .emacs.
;;
;;; FIXME: Allow customization for these. (i.e. defcustom).
(defvar template-mode-string "Tmpl"
  "String to identify template mode in the modeline.")
(defvar template-prefix-key "\C-ct"
  "Prefix key (default: C-c t) to be used for template mode commands.")

(add-to-list 'minor-mode-alist (list 'template-mode template-mode-string))

(defvar template-mode-map (make-sparse-keymap))
(defvar template-prefix-map (make-sparse-keymap)
  "Keymap for template mode. These key bindings are active after
template-prefix-key is hit")

(defun template-mode (&optional arg)
  "Toggle template mode in the current buffer. NOTE: Turning off template
Mode might be broken."
  (interactive "P")
  (if (if arg (> (prefix-numeric-value arg) 0) (not template-mode))
      (setq template-mode t)
    (setq template-mode nil))

  (or (assq 'template-mode minor-mode-alist)
      (setq minor-mode-alist
            (cons '(template-mode "Tmpl") minor-mode-alist)))
  )

(defun template-file-path (file)
  (concat template-files-dir file))


;;; This was stolen from:
;;; http://www.han.de/~racke/ginel/
;;; ginel-string.el --- GINEL string manipulation functions
;; Copyright (C) 1998 Stefan Hornburg
;; Author: Stefan Hornburg <racke@gundel.han.de>
;; Maintainer: Stefan Hornburg <racke@gundel.han.de>
;; Version: 0.2.0
;; Keywords: 
;; (under the terms of the GPL)

(defun string-replace (string regexp replacement)
  "Replaces all occurences of REGEXP within STRING by REPLACEMENT."
  (save-match-data 
    (let ((pos 0)
          (newstring ""))
      (while (string-match regexp string pos) 
        (setq newstring (concat newstring 
                                (substring string pos (match-beginning 0))
                                replacement))
        (setq pos (match-end 0)))
      (concat newstring (substring string pos)))))

;; This function takes a filename and a list of prompts. The filename
;; is found in 'template-files-dir' and inserted into the current buffer
;; at point. For each prompt in prompt-list, the user is prompted to
;; enter a string in the minibuffer. The user's strings are then used to
;; replace text in the inserted file. "xxx_replacement_N_xxx" is the text
;; that will be replaced, where N corresponds (starting with 0) to the
;; position in prompt-list. Note that all occurrences of
;; xxx_replacement_N_xxx in the current buffer will be replaced. That is
;; why xxx_replacement_n_xxx was chosen: it is unlikely to occur.
(defun template-insert (filename prompt-list)
  (insert-file-contents (template-file-path filename))
  (setq n 0)
  (while (> (length prompt-list) n)
    (let* ((prompt (nth n prompt-list))
           (replacement-text (read-from-minibuffer (concat prompt ": ")))
           (placeholder-string (concat "xxx_placeholder_"
                                       (int-to-string n) "_xxx")))
      (save-excursion
        (replace-string placeholder-string replacement-text)))
    (setq n (+ n 1)))
  (let* ((filename (file-name-nondirectory (buffer-file-name)))
         (filestem (string-replace filename "\.[a-zA-Z]+$" "")))
    (save-excursion
      (replace-string "xxx_filename_xxx" filename))
    (save-excursion
      (replace-string "xxx_filestem_xxx" filestem)))
  (replace-string "xxx_start_here_xxx" ""))

;;
;; Set up the keymaps.
;;
(define-prefix-command 'template-prefix-map)
(define-key template-mode-map template-prefix-key 'template-prefix-map)

(add-to-list 'minor-mode-map-alist (cons 'template-mode template-mode-map))

;;
;; These variables control where the template files live.
;;

;;; NOTE: template-files-dir MUST end in a '/'!
;;; FIXME: Make these customizable.
(defvar template-files-dir "~/.emacs-lisp/templates/")

;;; 
;;; The format is:
;;; (key template-file-name prompt menu-desc)
;;;
;;; where:
;;; - key is the key to be used for this template (after the prefix)
;;; - template-file-name is the filename in the template-files-dir
;;; - prompt is how we will prompt the user for replacement text
;;;   (if prompt is "filename" then the name of the buffer being
;;;   modified will automatically be inserted instead of prompting the
;;;   user; "filestem" is the filename without its extension)
;;; - menu-desc is not currently used but is reserved for use in
;;;   creating menu items in a later release
;;;
;;; FIXME:
;;;  1. multiple replacements
;;; 
(defvar template-func-list
      '(("f" "template-func.c"  '("Function name") "C function definition")
        ("b" "template-body.c"  '("Function name" "Return Value") "C function declaration")
        ("c" "class.cpp"        '("Class name") "Class declaration")
        ("m" "member-func.cpp"  '("Function name") "Class member declaration")
        ("H" "template.h"       '() "C header file")
        ("F" "template.c"       '() "C source file")))

;(setq pl '("Function name" "Return Value"))
;(mapcar (lambda (s) (concat "s" s ": \n")) pl)

(defun template-set-keybindings ()
  (interactive)
  (setq n 0)
  (while (> (length template-func-list) n)
    (let* ((template-func (nth n template-func-list))
           (key (nth 0 template-func))
           (sym (intern (concat "template-command-func" (int-to-string n))))
           (file (nth 1 template-func))
           (prompt-list (nth 2 template-func))
	   (menu-name (nth 3 template-func))
           ;(interactive-args (car (mapcar (lambda (s) (concat "s" s ": \n"))
           ;prompt-list)))
           ;(interactive-funcall (list 'interactive interactive-args))
           (template-insert-funcall (list
                                     'template-insert file prompt-list)))
      (set sym nil)
      (fset sym
            (eval (list 'lambda '()
                        '(interactive)
                        template-insert-funcall)))
      (define-key template-prefix-map key sym))
    (setq n (+ n 1))))

(template-set-keybindings)

(provide 'template-mode)
